﻿Imports System.IO
Imports System.Math

Public Class Form1
    'Simple PC interface to use serial pic for data capture and control
    'Initial version in Microsoft Visual Basic 2008 Express Paul McMahon May 2009
    'Copyright 2009 - Paul McMahon VK3DIP email pmvk3dip@bigpond.net.au
    'Version 0.1 - Initial version May 2009
    'Version 0.2 - Added logging 7/5/2009
    'Version 0.3 - made all commands upper case, all hex lower
    'Version 0.4 - re-arranged adhoc command line added recent
    'Version 0.5 - bug fixes & add prescale by 100 to Voltages
    'Version 0.6 - better serial error handling and fixed refresh button
    'Version 0.7 - cut down frequency to 3 bytes
    'Version 0.8 - Rearranged and added scripts
    'Version 0.9 - Changed multipliers for voltages into divisors and set new defaults 16/7/2009
    ' 

    Dim MyVersion As String = "0.9"

#Region "Globals and flags"
    Dim MyV(2) As Integer ' array to hold current V values
    Dim MyNib(9) As Integer ' array of raw nibbles in
    Dim MyFreq As Long = 0 ' Value of frequency
    Dim MyGT As Single = 1.0 ' Value of gate time for current reading 1 or 0.1 seconds
    Dim Myi2cAddr As Byte ' Value of i2c address read
    Dim Myi2cVal As Byte ' Value of i2c data byte read
    Dim MyPosn As Integer = 0 ' What nibble are we up to
    Dim MyType As Integer = 0 ' What type of response is it, 0 = none yet, 1 = Voltage, 2 = Frequency, 3 = i2c read
    Dim MyTouched As Boolean = False ' set true if anything is changed to prompt for save
    Dim MyXoff As Boolean = True ' manual version of additional flow control, if true don't try and send more to interface.
    Dim MyLogOn As Boolean = False ' set if log to file on
    Dim MydbV0 As Boolean = False ' set if V0 changed.
    Dim MydbV1 As Boolean = False ' set if V1 changed.
    Dim MydbV2 As Boolean = False ' set if V2 changed.
    Dim MydbV1V2 As Boolean = False ' set if V1 or V2 changed.
    Dim MydbFreq As Boolean = False ' set if Frequency changed.
    Dim Mydbi2c As Boolean = False ' set if i2c received changed.
    Dim ScriptVar As String = "" ' Result of a user prompt in script
    Dim ScriptQues As Boolean = True ' Result of user question in script
    Const TimeoutError As Integer = 1
    Const UnknownError As Integer = 99
    Const BadConfigCommand As Integer = 2
    Const BadScriptCommand As Integer = 3
    Const ScriptQuit As Integer = 4

#End Region

#Region "Form Startup & Exit"

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' set version
        Me.Text = "VK3DIP Generic Interface V" & MyVersion
        ' preload the available display types and default to option 0

        ddV0type.Items.Add("Volts            (V)") '0
        ddV0type.Items.Add("Amps            ( V)") '1
        ddV0type.Items.Add("dB            (logV)") '2
        ddV0type.Items.Add("dB               (V)") '3
        ddV0type.Items.Add("Watts          (V*V)") '4
        ddV1type.Items.Add("Volts            (V)") '0
        ddV1type.Items.Add("Amps             (V)") '1
        ddV1type.Items.Add("dB            (logV)") '2
        ddV1type.Items.Add("dB               (V)") '3
        ddV1type.Items.Add("Watts          (V*V)") '4
        ddV2type.Items.Add("Volts            (V)") '0
        ddV2type.Items.Add("Amps             (V)") '1
        ddV2type.Items.Add("dB            (logV)") '2
        ddV2type.Items.Add("dB               (V)") '3
        ddV2type.Items.Add("Watts          (V*V)") '4
        ddV1V2type.Items.Add("Volts        (V1-V2)") '0
        ddV1V2type.Items.Add("Watts        (V1*V2)") '1
        ddV1V2type.Items.Add("SWR           (SWR1)") '2
        ddV1V2type.Items.Add("Refl Coeff    (REF1)") '3
        ddFreqtype.Items.Add("Frequency      (MHz)") '0
        ddFreqtype.Items.Add("Frequency       (Hz)") '1
        ddFreqtype.Items.Add("Frequency      (GHz)") '2
        ddi2ctype.Items.Add("Addr&Byte   (Hex/Bin)") '0
        ddi2ctype.Items.Add("Addr&Byte   (Hex/Hex)") '1
        ddi2ctype.Items.Add("Addr&Byte   (Dec/Bin)") '2
        ddBaud.Items.Add("4800")
        ddBaud.Items.Add("9600")
        ddBaud.Items.Add("19200")
        ddBaud.Items.Add("38400")
        ddBaud.Items.Add("57600")
        ddBaud.Items.Add("115200")


        ddV0type.SelectedIndex = 3 ' default to dB display on V0
        ddV1type.SelectedIndex = 0
        ddV2type.SelectedIndex = 0
        ddV1V2type.SelectedIndex = 0
        ddFreqtype.SelectedIndex = 0
        ddi2ctype.SelectedIndex = 0
        ddBaud.SelectedIndex = 2 ' default to 19200

        ' preload the available comm ports and set to first found
        ' or error msg if none found

        PortSelect.Items.Clear()
        For Each portName As String In My.Computer.Ports.SerialPortNames
            PortSelect.Items.Add(portName)
        Next

        Try
            PortSelect.SelectedIndex = 0
            btnCom.Text = "Start"
        Catch es As Exception
            btnCom.Text = "OOPS"
            MsgBox("No free ports found")
        End Try
        'Update the rest of the form
        MyTouched = False
        Call MyDisplayIt()
    End Sub
  
    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        Application.Exit()
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If MyTouched Then
            If MsgBox("Unsaved config changes, OK = exit application, Cancel = abort exit", MsgBoxStyle.OkCancel) = MsgBoxResult.Cancel Then
                e.Cancel = True
                Exit Sub
            End If
        End If
        Call stopit()
        Call EndLog()

    End Sub
#End Region

#Region "Configuration SAVE and LOAD"

    Private FileName As String
    'declaring filename that will be selected
    Dim sr As StreamReader    'streamreader is used to read text
    Dim sw As StreamWriter


    Private Sub LoadConfigToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LoadConfigToolStripMenuItem.Click
        Dim mylinein As String
        Dim myresult As Integer = 0
        Try
            With OpenFileDialog1
                .Title = "Load Configuration"
                .DefaultExt = "gic"
                .Filter = "Gen. Int. Config file (*.gic)|*.gic"
                'setting filters so that Text files and All Files choice appears in the Files of Type box
                'in the dialog
                If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                    'showDialog method makes the dialog box visible at run time
                    FileName = .FileName
                    sr = New StreamReader(.OpenFile)
                    'using streamreader to read the opened text file
                    mylinein = sr.ReadLine()
                    Do Until mylinein Is Nothing
                        If mylinein.Substring(0, 1) = "*" Then myresult = DoScriptLine(mylinein)
                        If myresult > 0 Then Exit Do ' some problem give up and finish
                        mylinein = sr.ReadLine()
                    Loop
                End If
            End With
        Catch es As Exception
            MessageBox.Show(es.Message)
        Finally
            If Not (sr Is Nothing) Then
                sr.Close()
            End If
        End Try
    End Sub


    Private Sub SaveConfigToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveConfigToolStripMenuItem.Click
        'streamwriter is used to write text     
        Try
            With SaveFileDialog1
                .Title = "Save Configuration"
                .FileName = FileName
                .DefaultExt = "gic"
                .Filter = "Gen. Int. Config file (*.gic)|*.gic"
                If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                    FileName = .FileName
                    sw = New StreamWriter(FileName)
                    sw.WriteLine("; Configuration Saved" & Format(Now, "F"))
                    sw.WriteLine("; Enabled")
                    sw.WriteLine("*V0C=" & cbV0.Checked)
                    sw.WriteLine("*V1C=" & cbV1.Checked)
                    sw.WriteLine("*V2C=" & cbV2.Checked)
                    sw.WriteLine("*V3C=" & cbV1V2.Checked)
                    sw.WriteLine("*FRC=" & cbFreq.Checked)
                    sw.WriteLine("*FFC=" & cbFreqfast.Checked)
                    sw.WriteLine("*I2C=" & cbi2c.Checked)
                    sw.WriteLine("; Labels")
                    sw.WriteLine("*V0L=" & txtV0lbl.Text)
                    sw.WriteLine("*V1L=" & txtV1lbl.Text)
                    sw.WriteLine("*V2L=" & txtV2lbl.Text)
                    sw.WriteLine("*V3L=" & txtV1V2lbl.Text)
                    sw.WriteLine("*FRL=" & txtFreqlbl.Text)
                    sw.WriteLine("*I2L=" & txti2clbl.Text)
                    sw.WriteLine("; Multipliers")
                    sw.WriteLine("*V0M=" & numV0mult.Value)
                    sw.WriteLine("*V1M=" & numV1mult.Value)
                    sw.WriteLine("*V2M=" & numV2Mult.Value)
                    sw.WriteLine("*V3M=" & numV1V2mult.Value)
                    sw.WriteLine("*FRM=" & numFreqmult.Value)
                    sw.WriteLine("*I2P=" & numi2cRport.Value)
                    sw.WriteLine("; Offsets")
                    sw.WriteLine("*V0O=" & numV0off.Value)
                    sw.WriteLine("*V1O=" & numV1off.Value)
                    sw.WriteLine("*V2O=" & numV2off.Value)
                    sw.WriteLine("*V3O=" & numV1V2off.Value)
                    sw.WriteLine("*FRO=" & numFreqoff.Value)
                    sw.WriteLine("; Types") ' 
                    sw.WriteLine("*V0T=" & ddV0type.Text)
                    sw.WriteLine("*V1T=" & ddV1type.Text)
                    sw.WriteLine("*V2T=" & ddV2type.Text)
                    sw.WriteLine("*V3T=" & ddV1V2type.Text)
                    sw.WriteLine("*FRT=" & ddFreqtype.Text)
                    sw.WriteLine("*I2T=" & ddi2ctype.Text)

                    MsgBox("Data saved", MsgBoxStyle.Information)
                    MyTouched = False
                End If
            End With
        Catch es As Exception
            MessageBox.Show(es.Message)
        Finally
            If Not (sw Is Nothing) Then
                sw.Close()
            End If
        End Try
    End Sub
#End Region

#Region "On Change"
    Private Sub cbV0_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbV0.CheckedChanged
        MyTouched = True
    End Sub

    Private Sub txtV0lbl_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtV0lbl.TextChanged
        MyTouched = True
    End Sub

    Private Sub numV0mult_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV0mult.ValueChanged
        MyTouched = True

    End Sub

    Private Sub numV0off_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV0off.ValueChanged
        MyTouched = True

    End Sub

    Private Sub ddV0type_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddV0type.SelectedIndexChanged
        MyTouched = True

    End Sub

    Private Sub cbV1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbV1.CheckedChanged
        MyTouched = True

    End Sub

    Private Sub txtV1lbl_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtV1lbl.TextChanged
        MyTouched = True

    End Sub

    Private Sub numV1mult_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV1mult.ValueChanged
        MyTouched = True

    End Sub

    Private Sub numV1off_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV1off.ValueChanged
        MyTouched = True

    End Sub

    Private Sub ddV1type_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddV1type.SelectedIndexChanged
        MyTouched = True

    End Sub

    Private Sub cbV2_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbV2.CheckedChanged
        MyTouched = True

    End Sub

    Private Sub txtV2lbl_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtV2lbl.TextChanged
        MyTouched = True

    End Sub

    Private Sub numV2Mult_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV2Mult.ValueChanged
        MyTouched = True

    End Sub

    Private Sub numV2off_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV2off.ValueChanged
        MyTouched = True

    End Sub

    Private Sub ddV2type_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddV2type.SelectedIndexChanged
        MyTouched = True

    End Sub

    Private Sub cbV1V2_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbV1V2.CheckedChanged
        MyTouched = True

    End Sub

    Private Sub txtV1V2lbl_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtV1V2lbl.TextChanged
        MyTouched = True

    End Sub

    Private Sub numV1V2mult_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV1V2mult.ValueChanged
        MyTouched = True

    End Sub

    Private Sub numV1V2off_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numV1V2off.ValueChanged
        MyTouched = True

    End Sub

    Private Sub cbFreq_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbFreq.CheckedChanged
        MyTouched = True

    End Sub

    Private Sub txtFreqlbl_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtFreqlbl.TextChanged
        MyTouched = True

    End Sub

    Private Sub cbFreqfast_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbFreqfast.CheckedChanged
        MyTouched = True

    End Sub

    Private Sub numFreqmult_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numFreqmult.ValueChanged
        MyTouched = True

    End Sub

    Private Sub numFreqoff_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numFreqoff.ValueChanged
        MyTouched = True

    End Sub

    Private Sub ddFreqtype_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddFreqtype.SelectedIndexChanged
        MyTouched = True

    End Sub

    Private Sub cbi2c_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbi2c.CheckedChanged
        MyTouched = True

    End Sub

    Private Sub txti2clbl_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txti2clbl.TextChanged
        MyTouched = True

    End Sub

    Private Sub numi2cRport_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numi2cRport.ValueChanged
        MyTouched = True

    End Sub

    Private Sub ddi2ctype_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddi2ctype.SelectedIndexChanged
        MyTouched = True

    End Sub

    Private Sub ddV1V2type_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddV1V2type.SelectedIndexChanged
        MyTouched = True

    End Sub
#End Region

#Region "Update form"
    Private Sub MyDisplayIt()
        Dim tempvar As Double
        Dim tempvar2 As Double
        'V0
        If MydbV0 Then ' if new data try and log it
            ' Default divisor of 10 assumes ad8307 at 25mv per dB and 2.56 volt = 1024 => /10 gives approx dB
            tempvar = CDbl(MyV(0))
            Select Case ddV0type.SelectedIndex
                Case 0 'Straight volts
                    tempvar = tempvar / CDbl(numV0mult.Value) + CDbl(numV0off.Value)
                    txtV0.Text = Format(tempvar, "###0.00") & " Volts"
                Case 1 'Straight Amps
                    tempvar = tempvar / CDbl(numV0mult.Value) + CDbl(numV0off.Value)
                    txtV0.Text = Format(tempvar, "###0.00") & " Amps"
                Case 2 'Calculated dB
                    tempvar = Log(tempvar, 10) / CDbl(numV0mult.Value) + CDbl(numV0off.Value)
                    txtV0.Text = Format(tempvar, "###0.00") & "  dB"
                Case 3 'Straight dB
                    tempvar = tempvar / CDbl(numV0mult.Value) + CDbl(numV0off.Value)
                    txtV0.Text = Format(tempvar, "###0.00") & " dB"
                Case 4 'Calculated Power
                    tempvar = tempvar * tempvar / CDbl(numV0mult.Value) + CDbl(numV0off.Value)
                    txtV0.Text = Format(tempvar, "###0.00") & " Watts"
            End Select
            LogIt("V0", txtV0lbl.Text, txtV0.Text)
            MydbV0 = False
        End If
        'V1
        If MydbV1 Then ' if new data try and log it
            tempvar = CDbl(MyV(1)) 'default /102.3  assumes  1023 = 10 volts
            Select Case ddV1type.SelectedIndex
                Case 0 'Straight volts
                    tempvar = tempvar / CDbl(numV1mult.Value) + CDbl(numV1off.Value)
                    txtV1.Text = Format(tempvar, "###0.00") & " Volts"
                Case 1 'Straight Amps
                    tempvar = tempvar / CDbl(numV1mult.Value) + CDbl(numV1off.Value)
                    txtV1.Text = Format(tempvar, "###0.00") & " Amps"
                Case 2 'Calculated dB
                    tempvar = Log(tempvar, 10) / CDbl(numV1mult.Value) + CDbl(numV1off.Value)
                    txtV1.Text = Format(tempvar, "###0.00") & "  dB"
                Case 3 'Straight dB
                    tempvar = tempvar / CDbl(numV1mult.Value) + CDbl(numV1off.Value)
                    txtV1.Text = Format(tempvar, "###0.00") & " dB"
                Case 4 'Calculated Power
                    tempvar = tempvar * tempvar / CDbl(numV1mult.Value) + CDbl(numV1off.Value)
                    txtV1.Text = Format(tempvar, "###0.00") & " Watts"
            End Select
            LogIt("V1", txtV1lbl.Text, txtV1.Text)
            MydbV1 = False
        End If

        'V2
        If MydbV2 Then ' if new data try and log it
            tempvar = CDbl(MyV(2)) 'default /102.3  assumes  1023 = 10 volts
            Select Case ddV2type.SelectedIndex
                Case 0 'Straight volts
                    tempvar = tempvar / CDbl(numV2Mult.Value) + CDbl(numV2off.Value)
                    txtV2.Text = Format(tempvar, "###0.00") & " Volts"
                Case 1 'Straight Amps
                    tempvar = tempvar / CDbl(numV2Mult.Value) + CDbl(numV2off.Value)
                    txtV2.Text = Format(tempvar, "###0.00") & " Amps"
                Case 2 'Calculated dB
                    tempvar = Log(tempvar, 10) / CDbl(numV2Mult.Value) + CDbl(numV2off.Value)
                    txtV2.Text = Format(tempvar, "###0.00") & "  dB"
                Case 3 'Straight dB
                    tempvar = tempvar / CDbl(numV2Mult.Value) + CDbl(numV2off.Value)
                    txtV2.Text = Format(tempvar, "###0.00") & " dB"
                Case 4 'Calculated Power
                    tempvar = tempvar * tempvar / CDbl(numV2Mult.Value) + CDbl(numV2off.Value)
                    txtV2.Text = Format(tempvar, "###0.00") & " Watts"
            End Select
            LogIt("V2", txtV2lbl.Text, txtV2.Text)
            MydbV2 = False
        End If

        'V1 & V2
        If MydbV1V2 Then ' if new data try and log it
            tempvar = CDbl(MyV(1)) 'default /102.3  assumes  1023 = 10 volts
            tempvar2 = CDbl(MyV(2))

            Select Case ddV1V2type.SelectedIndex
                Case 0 'Diff volts
                    tempvar = (tempvar - tempvar2) / CDbl(numV1V2mult.Value) + CDbl(numV1V2off.Value)
                    txtV1V2.Text = Format(tempvar, "###0.00") & "  Volts"
                Case 1 'Power
                    tempvar = tempvar * tempvar2 / CDbl(numV1V2mult.Value) + CDbl(numV1V2off.Value)
                    txtV1V2.Text = Format(tempvar, "###0.00") & "  Watts"
                Case 2 'SWR, V1 = Vf , V2 = Vr
                    Try
                        tempvar = (CDbl(MyV(2)) / CDbl(MyV(1)))
                        If tempvar > 1 Or MyV(2) = 0 Then
                            txtV1V2.Text = "Invalid Values"
                        Else
                            txtV1V2.Text = Format(((1 + tempvar) / (1 - tempvar)) / numV1V2mult.Value + numV1V2off.Value, "###0.00") & " : 1"
                        End If
                    Catch ex As Exception
                        ' do nothing assume just invalid values
                        txtV1V2.Text = "Invalid Values"

                    End Try
                Case 3 'Reflection Coeff V1 = Vf , V2 = Vr
                    Try
                        tempvar = (CDbl(MyV(2)) / CDbl(MyV(1)))
                        If tempvar > 1 Or MyV(2) = 0 Then
                            txtV1V2.Text = "Invalid Values"
                        Else
                            txtV1V2.Text = Format(tempvar / numV1V2mult.Value + numV1V2off.Value, "###0.00")
                        End If

                    Catch ex As Exception
                        ' do nothing assume just invalid values
                        txtV1V2.Text = "Invalid Values"
                    End Try
            End Select
            LogIt("V1V2", txtV1V2lbl.Text, txtV1V2.Text)
            MydbV1V2 = False
        End If

        'Frequency
        If MydbFreq Then ' if new data try and log it
            Try
                tempvar = CDbl(MyFreq) / CDbl(MyGT) ' Default assume 64 by prescaler
                Select Case ddFreqtype.SelectedIndex
                    Case 0 'MHz
                        txtFreq.Text = Format((tempvar / 1000000) * numFreqmult.Value + numFreqoff.Value, "###0.000000") & "  MHz"
                    Case 1 'Hz
                        txtFreq.Text = Format(tempvar * numFreqmult.Value + numFreqoff.Value, "###0.0") & "  Hz"
                    Case 2 'GHz
                        txtFreq.Text = Format((tempvar / 1000000000) * numFreqmult.Value + numFreqoff.Value, "###0.000000") & "  GHz"
                End Select
            Catch ex As Exception
                ' do nothing assume just invalid values
                txtFreq.Text = ""
            End Try
            LogIt("Freq", txtFreqlbl.Text, txtFreq.Text)
            MydbFreq = False
        End If

        'i2c
        If Mydbi2c Then ' if new data try and log it
            Select Case ddi2ctype.SelectedIndex
                Case 0 'Port addr in Hex, recv byte in Bin
                    txti2c.Text = "ADDR " & BInt2Hex(Myi2cAddr) & "=" & Byte2Bin(Myi2cVal)
                Case 1 'Port addr in Hex, recv byte in Hex
                    txti2c.Text = "ADDR " & BInt2Hex(Myi2cAddr) & "=" & BInt2Hex(Myi2cVal)
                Case 2 'Port addr in Dec, recv byte in Bin
                    txti2c.Text = "ADDR " & Format(Myi2cAddr, "00") & "=" & Byte2Bin(Myi2cVal)
            End Select
            LogIt("i2cR", txti2clbl.Text, txti2c.Text)
            Mydbi2c = False
        End If

        ' flow led
        If MyXoff Then
            LabLed.BackColor = Color.Red
        Else
            LabLed.BackColor = Color.GreenYellow
        End If
    End Sub

#End Region

#Region "Commport Select"
    Private Sub portselect_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
        If SerialPort1.IsOpen Then
            SerialPort1.Close()
            btnCom.Text = "Start"
            Timer1.Enabled = False
            MyXoff = True
        End If
    End Sub
    Private Sub ddBaud_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddBaud.SelectedIndexChanged
        If SerialPort1.IsOpen Then
            Call stopit()
        End If
    End Sub
    Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click

        If SerialPort1.IsOpen Then
            Call stopit()
        End If

        PortSelect.Items.Clear()
        For Each portName As String In My.Computer.Ports.SerialPortNames
            PortSelect.Items.Add(portName)
        Next
        Try
            PortSelect.SelectedIndex = 0
            btnCom.Text = "Start"
        Catch es As Exception
            btnCom.Text = "OOPS"
            MessageBox.Show("No free ports found")
        End Try
    End Sub
#End Region

#Region "Serial Comms Handler"
    'sub to handle the serial port data received event
    Private Sub DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived
        'invoke textbox with delegate to cross from serial tread to form thread
        txtSerIn.Invoke(New myDelegate(AddressOf MySerIn), New Object() {})
    End Sub
    ' delegate and character in handler
    Public Delegate Sub myDelegate()
    Public Sub MySerIn()
        Dim MyText As String
        Dim MyChar As String
        Dim MyTempChar As String
        Dim Mycnt As Integer
        MyText = SerialPort1.ReadExisting
        For Mycnt = 1 To Len(MyText)
            MyChar = Mid(MyText, Mycnt, 1)
            Select Case MyChar
                Case Chr(19)
                    'its an Xoff
                    MyXoff = True
                Case Chr(17)
                    'its an Xon
                    MyXoff = False
                Case "V" ' Its going to be a voltage
                    MyType = 1
                    MyPosn = 0
                Case "F" 'Its going to be a Frequency
                    MyType = 2
                    MyFreq = 0
                    MyPosn = 0
                Case "R" ' Its going to be a i2c read
                    MyType = 3
                    MyPosn = 0
                Case "0" To "9"
                    If MyPosn < 10 Then
                        MyNib(MyPosn) = Val(MyChar)
                        MyPosn = MyPosn + 1
                    Else ' too much data
                        ' its an error just clean up
                        MyType = 0
                        MyPosn = 0
                    End If
                Case "a" To "f" ' Hex must use lower case
                    If MyPosn < 10 Then
                        MyTempChar = "&H" & MyChar
                        MyNib(MyPosn) = Val(MyTempChar)
                        MyPosn = MyPosn + 1
                    Else ' too much data
                        ' its an error just clean up
                        MyType = 0
                        MyPosn = 0
                    End If
                Case Chr(10) ' Got new line time to assemble it
                    Select Case MyType
                        Case 1 ' its a voltage
                            'first 2 nibbles = voltage num, first nibble always 0 second 0 to 2
                            'next four nibbles = value
                            Select Case MyPosn
                                Case 6 ' we have normal 6 ( 0 to 5) nibbles
                                    Select Case MyNib(1)
                                        Case 0
                                            MyV(0) = MyNib(2) * 4096 + MyNib(3) * 256 + MyNib(4) * 16 + MyNib(5) * 1
                                            MydbV0 = True ' set flag so we know to log the new value
                                        Case 1
                                            MyV(1) = MyNib(2) * 4096 + MyNib(3) * 256 + MyNib(4) * 16 + MyNib(5) * 1
                                            MydbV1 = True ' set flag so we know to log the new value
                                            MydbV1V2 = True
                                        Case 2
                                            MyV(2) = MyNib(2) * 4096 + MyNib(3) * 256 + MyNib(4) * 16 + MyNib(5) * 1
                                            MydbV2 = True ' set flag so we know to log the new value
                                            MydbV1V2 = True
                                        Case Else
                                            ' its an error just clean up
                                            MyType = 0
                                            MyPosn = 0
                                    End Select
                                Case Else
                                    ' its an error just clean up
                                    MyType = 0
                                    MyPosn = 0
                            End Select

                        Case 2 ' its a Frequency
                            'first 2 nibbles = gate time, first nibble 1 = 1 sec, second nibble 1 = 0.1 sec
                            'next eight nibbles = value
                            Select Case MyPosn
                                Case 8 ' we have normal 8 ( 0 to 7) nibbles
                                    If MyNib(0) = 1 Then
                                        MyGT = 1
                                    Else
                                        MyGT = 0.1
                                    End If
                                    MyFreq = MyNib(2) * 1048576 + MyNib(3) * 65536 + MyNib(4) * 4096 + MyNib(5) * 256 + MyNib(6) * 16 + MyNib(7) * 1
                                    MydbFreq = True ' set flag so we know to log the new value
                                Case Else
                                    ' its an error just clean up
                                    MyType = 0
                                    MyPosn = 0
                            End Select

                        Case 3 ' its a i2c read
                            'first 2 nibbles = Address
                            'next 2 nibbles = value
                            Select Case MyPosn
                                Case 4 ' we have normal 4 ( 0 to 3) nibbles
                                    Myi2cAddr = MyNib(0) * 16 + MyNib(1)
                                    Myi2cVal = MyNib(2) * 16 + MyNib(3)
                                    Mydbi2c = True ' set flag so we know to log the new value
                                Case Else
                                    ' its an error just clean up
                                    MyType = 0
                                    MyPosn = 0
                            End Select

                        Case Else
                            ' ignore it for now
                    End Select
                    Call MyDisplayIt()
            End Select
        Next
        If ReceivedToolStripMenuItem.Checked Then
            With txtSerIn
                .SelectionColor = Color.Red
                .AppendText(MyText)
                .ScrollToCaret()
            End With
        End If
    End Sub

    'For Send

    Private Sub txtSerOut_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtSerOut.KeyPress
        Dim myresult As Integer
        If e.KeyChar = Chr(13) Then ' send it
            myresult = Sendit(txtSerOut.Text & vbCrLf, 2)
            If myresult = 0 Then
                addifnotgot(txtSerOut.Text)
                txtSerOut.Text = ""
            End If
            e.Handled = True
        End If

    End Sub
    ' to Start & Stop
    Private Sub btnCom_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCom.Click
        If btnCom.Text = "OOPS" Then
            MsgBox("No free ports found")
        Else
            If btnCom.Text = "Start" Then
                Call startit()
            Else ' must be stop
                Call stopit()
            End If
        End If
    End Sub
    Private Sub stopit()
        If SerialPort1.IsOpen Then
            Try
                SerialPort1.Close()
                btnCom.Text = "Start"
                Timer1.Enabled = False
                MyXoff = True
                LogIt(PortSelect.Text, ddBaud.Text, "Stopped")
            Catch ex As Exception
                MsgBox(ex.ToString)
            End Try
        End If
    End Sub
    Private Sub startit()
        If SerialPort1.IsOpen Then
            SerialPort1.Close()
        End If
        Timer1.Enabled = False
        MyXoff = True
        Try
            With SerialPort1
                .PortName = PortSelect.Text
                .BaudRate = ddBaud.Text
                .Handshake = Ports.Handshake.XOnXOff
                .Parity = IO.Ports.Parity.None
                .DataBits = 8
                .StopBits = IO.Ports.StopBits.One
            End With
            SerialPort1.Open()
            ' all OK switch buttons, start poll and set flow.
            btnCom.Text = "Stop"
            Timer1.Enabled = True
            MyXoff = False
            LogIt(PortSelect.Text, ddBaud.Text, "Started")
        Catch ex As Exception
            MsgBox(ex.ToString)
        End Try
    End Sub

    Private Function Sendit(ByVal mystring As String, ByVal mytype As Integer) As Integer
        Dim mytimeout As Date ' Time for timeout of type 2&3
        Dim mycancel As Boolean = False
        Dim myresult As Integer = 0
        mytimeout = Now.AddSeconds(3) 'timeout in three seconds from now
        'mytype
        ' 1 = Poll stuff
        ' 2 = Manual
        ' 3 = Script 
        'Type 2 & 3 will wait for XON before sending rather than returning immediatly if XOFF,
        ' others if not flow controlled send it, if flow controlled then just ignore  
        If mytype = 1 Then
            If Not MyXoff Then
                Try
                    MyXoff = True 'stop any further writes until the interface responds
                    SerialPort1.Write(mystring)
                Catch ex As Exception
                    Call stopit()
                    MsgBox(ex.ToString)
                    myresult = UnknownError
                End Try
                Select Case mytype
                    Case 1 ' Poll
                        If PollToolStripMenuItem.Checked Then
                            With txtSerIn
                                .SelectionColor = Color.Green
                                .AppendText(mystring)
                                .ScrollToCaret()
                            End With
                        End If
                End Select
            End If
        Else
            While MyXoff
                ' wait for XON
                If mytimeout.CompareTo(Now) < 0 Then ' more than 3 seconds have elapsed waiting
                    MsgBox("Timed out waiting for interface")
                    mycancel = True
                    myresult = TimeoutError
                    MyXoff = False ' clear on timeout
                    Exit While
                End If
            End While
            If Not mycancel Then
                Try
                    MyXoff = True 'stop any further writes until the interface responds
                    SerialPort1.Write(mystring)
                Catch ex As Exception
                    Call stopit()
                    MsgBox(ex.ToString)
                    myresult = UnknownError
                End Try
                Select Case mytype
                    Case 2 ' Manual command
                        If SentToolStripMenuItem.Checked Then
                            With txtSerIn
                                .SelectionColor = Color.Black
                                .AppendText(mystring)
                                .ScrollToCaret()
                            End With
                        End If
                    Case 3 ' Script
                        If ScriptSentToolStripMenuItem.Checked Then
                            With txtSerIn
                                .SelectionColor = Color.RoyalBlue
                                .AppendText(mystring)
                                .ScrollToCaret()
                            End With
                        End If
                End Select
            End If

        End If

        Return myresult
    End Function
    Private Sub ddCmds_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddCmds.SelectedIndexChanged
        txtSerOut.Text = ddCmds.Text
    End Sub

#End Region

#Region "Poll"
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        Dim myresult As Integer
        Timer1.Enabled = False ' stop repeated triggers
        Call MyDisplayIt()
        'Do volts first
        If (cbV0.Checked And cbV1.Checked And cbV2.Checked) Or (cbV0.Checked And cbV1V2.Checked) Then
            myresult = Sendit("V03" & vbCrLf, 1)
        Else
            If cbV0.Checked Then
                myresult = Sendit("V00" & vbCrLf, 1)
            End If
            If cbV1V2.Checked Then
                myresult = Sendit("V01" & vbCrLf, 1)
                myresult = Sendit("V02" & vbCrLf, 1)
            Else
                If cbV1.Checked Then
                    myresult = Sendit("V01" & vbCrLf, 1)
                End If
                If cbV2.Checked Then
                    myresult = Sendit("V02" & vbCrLf, 1)
                End If
            End If
        End If
        'frequency
        If cbFreq.Checked Then
            If cbFreqfast.Checked Then
                myresult = Sendit("F01" & vbCrLf, 1)
            Else
                myresult = Sendit("F10" & vbCrLf, 1)
            End If
        End If
        'i2c
        If cbi2c.Checked Then
            myresult = Sendit("R" & BInt2Hex(numi2cRport.Value) & vbCrLf, 1)
        End If
        ' RS232 Signals
        If SerialPort1.CDHolding Then
            labCD.BackColor = Color.Green
        Else
            labCD.BackColor = Color.Red
        End If
        If SerialPort1.CtsHolding Then
            LabCTS.BackColor = Color.Green
        Else
            LabCTS.BackColor = Color.Red
        End If
        If SerialPort1.DsrHolding Then
            LabDSR.BackColor = Color.Green
        Else
            LabDSR.BackColor = Color.Red
        End If

        Timer1.Enabled = True ' re-enable
    End Sub
  
#End Region

#Region "Misc"

    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutToolStripMenuItem.Click
        MsgBox("VK3DIP Generic Interface Version " & MyVersion & vbCrLf & "Copyright Paul McMahon 2009" & vbCrLf & "email pmvk3dip@bigpond.net.au ")
    End Sub
    Public Function Byte2Bin(ByVal DecNum As Byte) As String
        'converts a byte into a string of binary digits
        Dim mystemp As String
        mystemp = "00000000" & Convert.ToString(DecNum, 2)
        mystemp = Microsoft.VisualBasic.Right(mystemp, 8)

        Return mystemp
    End Function
    Public Function BInt2Hex(ByVal DecNum As Integer) As String
        'converts a byte stored in an int into a two char hex string
        Dim mytemp As Byte
        Dim mystemp As String

        mytemp = DecNum
        mystemp = "00" & Convert.ToString(mytemp, 16).ToLower
        mystemp = Microsoft.VisualBasic.Right(mystemp, 2)
        Return mystemp
    End Function
    Private Sub addifnotgot(ByVal newstring As String)
        Dim i As Integer
        Dim myfound As Boolean = False

        i = ddCmds.FindStringExact(newstring)
        If i < 0 Then ddCmds.Items.Add(newstring)

    End Sub

    Private Sub PollToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PollToolStripMenuItem.Click
        PollToolStripMenuItem.Checked = Not PollToolStripMenuItem.Checked
    End Sub

    Private Sub SentToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SentToolStripMenuItem.Click
        SentToolStripMenuItem.Checked = Not SentToolStripMenuItem.Checked
    End Sub
    Private Sub ScriptSentToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ScriptSentToolStripMenuItem.Click
        ScriptSentToolStripMenuItem.Checked = Not ScriptSentToolStripMenuItem.Checked
    End Sub
    Private Sub ReceivedToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReceivedToolStripMenuItem.Click
        ReceivedToolStripMenuItem.Checked = Not ReceivedToolStripMenuItem.Checked
    End Sub

    Private Sub ClearToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ClearToolStripMenuItem.Click
        txtSerIn.Text = ""
    End Sub
#End Region

#Region "Logging"
    Private LogFileName As String = "Log"
    'declaring filename that will be selected
    Dim lsw As StreamWriter
    Private Sub StartLogToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartLogToolStripMenuItem.Click
        'streamwriter is used to write text     
        Try
            With SaveFileDialog1
                .Title = "Select Logfile"
                .FileName = LogFileName
                .DefaultExt = "gil"
                .Filter = "Gen. Int. Log file (*.gil)|*.gil"
                If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                    FileName = .FileName
                    lsw = New StreamWriter(FileName)
                    lsw.WriteLine("Log file started " & Format(Now, "F"))
                    StartLogToolStripMenuItem.Enabled = False
                    EndLogToolStripMenuItem.Enabled = True
                    MyLogOn = True
                End If
            End With
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try

    End Sub

    Private Sub EndLogToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndLogToolStripMenuItem.Click
        Call EndLog()

    End Sub
    Private Sub EndLog()
        If MyLogOn = True Then
            Try
                lsw.WriteLine("Log file stopped " & Format(Now, "F"))
                lsw.Close()
                StartLogToolStripMenuItem.Enabled = True
                EndLogToolStripMenuItem.Enabled = False
                MyLogOn = False
            Catch ex As Exception
                MessageBox.Show(ex.Message)

            End Try
        End If
    End Sub
    Private Sub LogIt(ByVal myitem As String, ByVal mylabel As String, ByVal myvalue As String)
        If MyLogOn Then
            lsw.WriteLine(Format(Now, "T") & " -> " & Chr(34) & myitem & Chr(34) & " , " & Chr(34) & mylabel & Chr(34) & " , " & myvalue)
        End If
    End Sub
#End Region

#Region "Scripts"
    Private ScriptFileName As String
    'declaring filename that will be selected
    Dim ssr As StreamReader

    Private Sub RunScriptToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RunScriptToolStripMenuItem.Click
        Dim mylinein As String
        Dim myresult As Integer = 0
        Try
            With OpenFileDialog1
                .Title = "Run Script"
                .DefaultExt = "gis"
                .Filter = "Gen. Int. Script file (*.gis)|*.gis"
                'setting filters so that Text files and All Files choice appears in the Files of Type box
                'in the dialog
                If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                    'showDialog method makes the dialog box visible at run time
                    FileName = .FileName
                    ssr = New StreamReader(.OpenFile)
                    'using streamreader to read the opened text file
                    LogIt("Script", "Opened", FileName)
                    mylinein = ssr.ReadLine()
                    Do Until mylinein Is Nothing
                        myresult = DoScriptLine(mylinein)
                        If myresult > 0 Then Exit Do ' some problem give up and finish
                        mylinein = ssr.ReadLine()
                    Loop
                    ssr.Close()
                    LogIt("Script", "Closed", FileName)
                End If

            End With
        Catch es As Exception
            MessageBox.Show(es.Message)
        Finally
            If Not (ssr Is Nothing) Then
                ssr.Close()
            End If
        End Try
    End Sub
    Private Function DoScriptLine(ByVal linein As String) As Integer
        Dim myresult As Integer = 0
        Select Case linein.Substring(0, 1)
            Case ";" 'Its a comment just add it to the log if enabled
                Call LogIt("Script", "Comment", linein)
            Case "*" 'Its a config item
                myresult = configit(linein)
                Call LogIt("Script", "Config", linein)
            Case "#" 'Its a script command
                Call LogIt("Script", "Command Start", linein)
                myresult = mydoit(linein)
                If myresult = 0 Then
                    Call LogIt("Script", "Cmd End", linein)
                Else
                    Call LogIt("Script", "Cmd Fail#" & myresult, linein)
                End If
            Case Else ' It is a command for the Interface just send it
                Call LogIt("Script", "To Inteface", linein)
                myresult = Sendit(linein & vbCrLf, 3) ' sendit type 3 will wait for xon rather than returning immediatly if XOFF
                If myresult <> 0 Then
                    Call LogIt("Script", "Send Fail#" & myresult, linein)
                End If
        End Select
        Return myresult
    End Function
    Private Function configit(ByVal mycmd As String) As Integer
        Dim myresult As Integer = 0
        Try
            Select Case mycmd.Substring(1, 1)
                Case "V" ' Volts
                    Select Case mycmd.Substring(2, 1)
                        Case "0"
                            Select Case mycmd.Substring(3, 1)
                                Case "C" ' checked
                                    cbV0.Checked = mycmd.Substring(5)
                                Case "L" ' Label
                                    txtV0lbl.Text = mycmd.Substring(5)
                                Case "M" 'Multiplier
                                    numV0mult.Value = mycmd.Substring(5)
                                Case "O" 'Offset
                                    numV0off.Value = mycmd.Substring(5)
                                Case "T" 'Type
                                    ddV0type.Text = mycmd.Substring(5)
                                Case Else
                                    ' its an error
                                    myresult = BadConfigCommand
                            End Select
                        Case "1"
                            Select Case mycmd.Substring(3, 1)
                                Case "C" ' checked
                                    cbV1.Checked = mycmd.Substring(5)
                                Case "L" ' Label
                                    txtV1lbl.Text = mycmd.Substring(5)
                                Case "M" 'Multiplier
                                    numV1mult.Value = mycmd.Substring(5)
                                Case "O" 'Offset
                                    numV1off.Value = mycmd.Substring(5)
                                Case "T" 'Type
                                    ddV1type.Text = mycmd.Substring(5)
                                Case Else
                                    ' its an error
                                    myresult = BadConfigCommand
                            End Select

                        Case "2"
                            Select Case mycmd.Substring(3, 1)
                                Case "C" ' checked
                                    cbV2.Checked = mycmd.Substring(5)
                                Case "L" ' Label
                                    txtV2lbl.Text = mycmd.Substring(5)
                                Case "M" 'Multiplier
                                    numV2Mult.Value = mycmd.Substring(5)
                                Case "O" 'Offset
                                    numV2off.Value = mycmd.Substring(5)
                                Case "T" 'Type
                                    ddV2type.Text = mycmd.Substring(5)
                                Case Else
                                    ' its an error
                                    myresult = BadConfigCommand
                            End Select

                        Case "3"
                            Select Case mycmd.Substring(3, 1)
                                Case "C" ' checked
                                    cbV1V2.Checked = mycmd.Substring(5)
                                Case "L" ' Label
                                    txtV1V2lbl.Text = mycmd.Substring(5)
                                Case "M" 'Multiplier
                                    numV1V2mult.Value = mycmd.Substring(5)
                                Case "O" 'Offset
                                    numV1V2off.Value = mycmd.Substring(5)
                                Case "T" 'Type
                                    ddV1V2type.Text = mycmd.Substring(5)
                                Case Else
                                    ' its an error
                                    myresult = BadConfigCommand
                            End Select

                        Case Else
                            ' its an error
                            myresult = BadConfigCommand
                    End Select
                Case "F" 'Frequency
                    Select Case mycmd.Substring(3, 1)
                        Case "C" ' checked
                            If mycmd.Substring(2, 1) = "R" Then
                                cbFreq.Checked = mycmd.Substring(5)
                            Else ' should be "F"
                                cbFreqfast.Checked = mycmd.Substring(5)
                            End If
                            cbV1V2.Checked = mycmd.Substring(5)
                        Case "L" ' Label
                            txtFreqlbl.Text = mycmd.Substring(5)
                        Case "M" 'Multiplier
                            numFreqmult.Value = mycmd.Substring(5)
                        Case "O" 'Offset
                            numFreqoff.Value = mycmd.Substring(5)
                        Case "T" 'Type
                            ddFreqtype.Text = mycmd.Substring(5)
                        Case Else
                            ' its an error
                            myresult = BadConfigCommand
                    End Select

                Case "I" 'I2C
                    Select Case mycmd.Substring(3, 1)
                        Case "C" ' checked
                            cbi2c.Checked = mycmd.Substring(5)
                        Case "L" ' Label
                            txti2clbl.Text = mycmd.Substring(5)
                        Case "P" 'Multiplier
                            numi2cRport.Value = mycmd.Substring(5)
                        Case "T" 'Type
                            ddi2ctype.Text = mycmd.Substring(5)
                        Case Else
                            ' its an error
                            myresult = BadConfigCommand
                    End Select

                Case Else
                    ' its an error
                    myresult = BadConfigCommand
            End Select
        Catch es As Exception
            MessageBox.Show(es.Message)
            myresult = BadConfigCommand
        End Try
        Return myresult
    End Function
    Private Function mydoit(ByVal mycmd As String) As Integer
        Dim myresult As Integer = 0
        Dim mytimeout As Date ' Time for delay
        Select Case mycmd.Substring(1, 1)
            Case "D" ' delay
                mytimeout = Now.AddSeconds(mycmd.Substring(2))
                While mytimeout.CompareTo(Now) > 0
                    ' wait for time to go hogging machine resources
                End While
            Case "W" ' Wait User
                MsgBox(mycmd.Substring(2))
            Case "P" ' Prompt User for Input into ScriptVar
                ScriptVar = InputBox(mycmd.Substring(2))
            Case "V" ' Do this line with ~ replaced with ScriptVar
                myresult = DoScriptLine(mycmd.Substring(2).Replace("~", ScriptVar))
            Case "Q" ' Question User yes = true, no = false in ScriptQues
                If MsgBox(mycmd.Substring(2), MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
                    ScriptQues = True
                Else
                    ScriptQues = False
                End If
            Case "Y" ' If ScriptQues then do this line
                If ScriptQues Then myresult = DoScriptLine(mycmd.Substring(2))
            Case "N" ' If not ScriptQues then do this line
                If Not ScriptQues Then myresult = DoScriptLine(mycmd.Substring(2))
            Case "F" ' Finished?
                If MsgBox("Do you want to continue?", MsgBoxStyle.YesNo) = MsgBoxResult.No Then myresult = ScriptQuit
            Case Else
                ' its an error
                myresult = BadScriptCommand
        End Select
        Return myresult

    End Function
#End Region


End Class

